<?php
// echo "test";
// die();
$entityObj = new Entity();

class Entity {

    var $mDb; 
    var $mConfig; 
    var $mMailer;

    function __construct() {
        global $Config; 
        $this->mDb = new iplus(); 
        $this->mConfig = $Config; 
        $this->mMailer = new Mailer();

    }
        function convert_object_to_array($data) {
        if (is_object($data)) {
            $data = get_object_vars($data);
        }
        
        if (is_array($data)) {
            return array_map(__METHOD__, $data);
        }
        else {
            // echo "test"; die();
            return $data;
        }
    }
    
 function getAllEntities($start, $perpage, $lang_code, $searchName = null, $type = null) {
    $result = array();

    $sql = "SELECT e.*, el.name, el.description, el.lang_code, u.full_name AS admin_name 
            FROM entities e
            LEFT JOIN entities_langs el ON el.entity_id = e.id AND el.lang_code = :lang_code
            LEFT JOIN users u ON e.added_by = u.id
            WHERE e.id > 0 "; 

    if ($searchName) {
        $sql .= " AND el.name LIKE :searchName"; 
    }

    if ($type) {
        $sql .= " AND e.type LIKE :type"; 
    }

    // Remove single quotes around limit placeholders
        $sql .= " LIMIT " . (int)$start . ", " . (int)$perpage;
// echo $sql; die();
    
        $stmt = $this->mDb->connect()->prepare($sql);
        // echo $sql;
        // die();

        if (!$stmt) {
            
            throw new Exception("Error preparing the SQL statement."); 
        }

        $stmt->bindParam(':lang_code', $lang_code, PDO::PARAM_STR);
        if ($searchName) {
            $searchName = '%' . $searchName . '%';
            $stmt->bindParam(':searchName', $searchName, PDO::PARAM_STR);
        }
        if ($type) {
            $type = '%' . $type . '%';
            $stmt->bindParam(':type', $type, PDO::PARAM_STR); 
        }

        // Make sure the limit parameters are bound as integers

        // $stmt->bindParam(':start', $start, PDO::PARAM_STR); 
        // $stmt->bindParam(':perpage', $perpage, PDO::PARAM_STR);
        
        if (!$stmt->execute()) { 
             
            throw new Exception("Error executing the SQL statement: " . implode(", ", $stmt->errorInfo()));
        }
        $result = $stmt->fetchAll(PDO::FETCH_ASSOC); // Fetch all results
//   print_r($result);
//   die();

    return $result;
}

function getEntitiesCount($searchName = null, $lang_code, $type = null) {
    $count = 0; 

    // 1. Build the SQL Query Dynamically (with placeholders)
    $sql = "SELECT COUNT(e.id) AS count 
            FROM entities e
            LEFT JOIN entities_langs el ON e.id = el.entity_id 
            WHERE e.id > 0 AND el.lang_code = :lang_code";

    if ($searchName) {
        $sql .= " AND el.name LIKE :searchName"; 
    }

    if ($type) {
        $sql .= " AND e.type LIKE :type";
    }

    // 2. Prepare and Execute with Error Handling
    try {
        $stmt = $this->mDb->connect()->prepare($sql);

        if (!$stmt) {
            throw new Exception("Error preparing the SQL statement for count.");
        }

        // Bind parameters (adding wildcards for LIKE)
        $stmt->bindParam(':lang_code', $lang_code, PDO::PARAM_STR);
        if ($searchName) {
            $searchName = '%' . $searchName . '%';
            $stmt->bindParam(':searchName', $searchName, PDO::PARAM_STR);
        }
        if ($type) {
            $type = '%' . $type . '%';
            $stmt->bindParam(':type', $type, PDO::PARAM_STR);
        }

        $stmt->execute();
        $count = (int)$stmt->fetchColumn(); // Fetch the count directly as an integer
    } catch (PDOException $e) {
        // Log the error for debugging
        throw new Exception("Database Error (Count): " . $e->getMessage());
    } catch (Exception $e) {
        // Log the error
        throw new Exception("Error (Count): " . $e->getMessage());
    }
    return $count;
}
    
    
function addEditEntity($request, $img, $entity_id)
{
    $categoryLangObj = json_decode($request['langs']);
    $newcategoryLangObj = $this->convert_object_to_array($categoryLangObj);
    $res = false;

    if (!$entity_id || $entity_id == "null" || $entity_id == null && $request['operation'] === 'add') {
        // Add
        unset($request['operation']);
        $dateTime = date('Y-m-d H:i:s');

        $sql = "INSERT INTO `entities` SET ";
        $sql .= " `added_by` = :user_id,";
        $sql .= $request['status'] === 'true' ? " `status` = '1'," : " `status` = '0',";
        $sql .= " `date_added` = :dateTime,";
        $sql .= " `sort` = :sort,";
        $sql .= " `img` = :img";

        $stmt = $this->mDb->connect()->prepare($sql);
        $stmt->bindParam(':user_id', $request['user_id'], PDO::PARAM_INT);
        $stmt->bindParam(':dateTime', $dateTime, PDO::PARAM_STR);
        $stmt->bindParam(':sort', $request['sort'], PDO::PARAM_INT);
        $stmt->bindParam(':img', $img, PDO::PARAM_STR);
        $stmt->execute();

        $last_entity_id = $this->mDb->getLastInsertId();

        foreach ($newcategoryLangObj as $key => $value) {
            $stmt = "INSERT INTO `entities_langs`(`entity_id`, `lang_code`, `name`, `description`) ";
            $stmt .= "VALUES (:entity_id, :lang_code, :name, :description)";

            $stmt = $this->mDb->connect()->prepare($stmt);
            $stmt->bindParam(':entity_id', $last_entity_id, PDO::PARAM_INT);
            $stmt->bindParam(':lang_code', $value['lang_code'], PDO::PARAM_STR);
            $stmt->bindParam(':name', $value['name'], PDO::PARAM_STR);
            $stmt->bindParam(':description', $value['description'], PDO::PARAM_STR);
            $stmt->execute();
        }

        return $stmt->rowCount() > 0;
    } else {
        // Edit
        $check_query = "SELECT `id` FROM `entities` WHERE `id` = :entity_id";
        $stmt = $this->mDb->connect()->prepare($check_query);
        $stmt->bindParam(':entity_id', $entity_id, PDO::PARAM_INT);
        $stmt->execute();
        $check_result = $stmt->fetch(PDO::FETCH_ASSOC);

        if (!$check_result) {
            return "edit";
        } else {
            $sql = "UPDATE `entities` SET ";
            $sql .= " `added_by` = :user_id, ";
            $sql .= " `sort` = :sort, ";
            $sql .= $request['status'] === 'true' ? " `status` = '1'" : " `status` = '0'";

            if ($img != '' || $img != null) {
                $img_query = "SELECT `img` FROM `entities` WHERE `id` = :entity_id";
                $stmt = $this->mDb->connect()->prepare($img_query);
                $stmt->bindParam(':entity_id', $entity_id, PDO::PARAM_INT);
                $stmt->execute();
                $img_result = $stmt->fetchColumn();

                $old_img = $_SERVER['DOCUMENT_ROOT'] . '/' . $this->mConfig['uploads_path'] . 'entities/' . $img_result;

                if (file_exists($old_img)) {
                    @unlink($old_img);
                }

                $sql .= ",`img`=:img ";
            }

            $sql .= " WHERE `id` = :entity_id";

            $stmt = $this->mDb->connect()->prepare($sql);
            $stmt->bindParam(':user_id', $request['user_id'], PDO::PARAM_INT);
            $stmt->bindParam(':sort', $request['sort'], PDO::PARAM_INT);
            $stmt->bindParam(':entity_id', $entity_id, PDO::PARAM_INT);
            $stmt->bindParam(':img', $img, PDO::PARAM_STR);
            $stmt->execute();

            foreach ($newcategoryLangObj as $key => $value) {
                $stmt = "UPDATE `entities_langs` SET";
                $stmt .= " `name`=:name, ";
                $stmt .= " `description`=:description ";
                $stmt .= " WHERE `lang_code`=:lang_code AND `entity_id`=:entity_id";

                $stmt = $this->mDb->connect()->prepare($stmt);
                $stmt->bindParam(':name', $value['name'], PDO::PARAM_STR);
                $stmt->bindParam(':description', $value['description'], PDO::PARAM_STR);
                $stmt->bindParam(':lang_code', $value['lang_code'], PDO::PARAM_STR);
                $stmt->bindParam(':entity_id', $entity_id, PDO::PARAM_INT);
                $stmt->execute();
            }

            return $stmt->rowCount() > 0;
        }
    }
}
function deleteEntity($id)
{
    $entity_id = '(' . implode(',', $id) . ')';
    
    $sql = "SELECT `img` FROM `entities`";
    $sql .= " WHERE `id` IN " . $entity_id;
    
    $stmt = $this->mDb->connect()->prepare($sql);
    $stmt->execute();
    $images = $stmt->fetchAll(PDO::FETCH_ASSOC);
    
    $sqll = "DELETE FROM `entities_langs` ";
    $sqll .= " WHERE `entity_id` IN " . $entity_id;
    
    $stmt = $this->mDb->connect()->prepare($sqll);
    $stmt->execute();
    
    $entitysql = "DELETE FROM `entities` ";
    $entitysql .= " WHERE `id` IN " . $entity_id;
    
    $stmt = $this->mDb->connect()->prepare($entitysql);
    $stmt->execute();
    
    if ($stmt->rowCount() > 0) {
        if (count($images) > 0) {
            foreach ($images as $image) {
                $old_img = $_SERVER['DOCUMENT_ROOT'] . '/' . $this->mConfig['uploads_path'] . $image['img'];
                if (file_exists($old_img)) {
                    @unlink($old_img);
                }
            }
        }
        return true;
    } else {
        return false;
    }
}

}
